"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Scope_1 = require("./api/Scope");
var InjectionToken_1 = require("./api/InjectionToken");
var Exception_1 = require("./Exception");
var DEFAULT_SCOPE = Scope_1.Scope.Singleton;
/*

# Composite design pattern:

         ┏━━━━━━━━━━━━━━━━━━┓
         ┃ AbstractInjector ┃
         ┗━━━━━━━━━━━━━━━━━━┛
                 ▲
                 ┃
          ┏━━━━━━┻━━━━━━━━━━━━━━━┳━━━━━━━━━━━━━━━━━━━━━━━━━━┓
          ┃                      ┃                          ┃
 ┏━━━━━━━━┻━━━━━┓   ┏━━━━━━━━━━━━┻━━━━━━━━━━━┓      ┏━━━━━━━┻━━━━━━━┓
 ┃ RootInjector ┃   ┃ AbstractCachedInjector ┃      ┃ ValueInjector ┃
 ┗━━━━━━━━━━━━━━┛   ┗━━━━━━━━━━━━━━━━━━━━━━━━┛      ┗━━━━━━━━━━━━━━━┛
                                ▲
                                ┃
                        ┏━━━━━━━┻━━━━━━━━━━━━┓
               ┏━━━━━━━━┻━━━━━━━━┓  ┏━━━━━━━━┻━━━━━━┓
               ┃ FactoryInjector ┃  ┃ ClassInjector ┃
               ┗━━━━━━━━━━━━━━━━━┛  ┗━━━━━━━━━━━━━━━┛
*/
var AbstractInjector = /** @class */ (function () {
    function AbstractInjector() {
    }
    AbstractInjector.prototype.injectClass = function (Class) {
        try {
            var args = this.resolveParametersToInject(Class);
            return new (Class.bind.apply(Class, [void 0].concat(args)))();
        }
        catch (error) {
            throw new Exception_1.default("Could not inject \"" + Class.name + "\"", error);
        }
    };
    AbstractInjector.prototype.injectFunction = function (fn, target) {
        try {
            var args = this.resolveParametersToInject(fn, target);
            return fn.apply(void 0, args);
        }
        catch (error) {
            throw new Exception_1.default("Could not inject \"" + fn.name + "\"", error);
        }
    };
    AbstractInjector.prototype.resolveParametersToInject = function (injectable, target) {
        var _this = this;
        var tokens = injectable.inject || [];
        return tokens.map(function (key) { return _this.resolve(key, target); });
    };
    AbstractInjector.prototype.provideValue = function (token, value) {
        return new ValueInjector(this, token, value);
    };
    AbstractInjector.prototype.provideClass = function (token, Class, scope) {
        if (scope === void 0) { scope = DEFAULT_SCOPE; }
        return new ClassInjector(this, token, scope, Class);
    };
    AbstractInjector.prototype.provideFactory = function (token, factory, scope) {
        if (scope === void 0) { scope = DEFAULT_SCOPE; }
        return new FactoryInjector(this, token, scope, factory);
    };
    AbstractInjector.prototype.resolve = function (token, target) {
        switch (token) {
            case InjectionToken_1.TARGET_TOKEN:
                return target;
            case InjectionToken_1.INJECTOR_TOKEN:
                return this;
            default:
                return this.resolveInternal(token);
        }
    };
    return AbstractInjector;
}());
var RootInjector = /** @class */ (function (_super) {
    tslib_1.__extends(RootInjector, _super);
    function RootInjector() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RootInjector.prototype.resolveInternal = function (token) {
        throw new Error("No provider found for \"" + token + "\"!.");
    };
    return RootInjector;
}(AbstractInjector));
var ValueInjector = /** @class */ (function (_super) {
    tslib_1.__extends(ValueInjector, _super);
    function ValueInjector(parent, token, value) {
        var _this = _super.call(this) || this;
        _this.parent = parent;
        _this.token = token;
        _this.value = value;
        return _this;
    }
    ValueInjector.prototype.resolveInternal = function (token, target) {
        if (token === this.token) {
            return this.value;
        }
        else {
            return this.parent.resolve(token, target);
        }
    };
    return ValueInjector;
}(AbstractInjector));
var AbstractCachedInjector = /** @class */ (function (_super) {
    tslib_1.__extends(AbstractCachedInjector, _super);
    function AbstractCachedInjector(parent, token, scope) {
        var _this = _super.call(this) || this;
        _this.parent = parent;
        _this.token = token;
        _this.scope = scope;
        return _this;
    }
    AbstractCachedInjector.prototype.resolveInternal = function (token, target) {
        if (token === this.token) {
            if (this.cached) {
                return this.cached.value;
            }
            else {
                var value = this.result(target);
                if (this.scope === Scope_1.Scope.Singleton) {
                    this.cached = { value: value };
                }
                return value;
            }
        }
        else {
            return this.parent.resolve(token, target);
        }
    };
    return AbstractCachedInjector;
}(AbstractInjector));
var FactoryInjector = /** @class */ (function (_super) {
    tslib_1.__extends(FactoryInjector, _super);
    function FactoryInjector(parent, token, scope, injectable) {
        var _this = _super.call(this, parent, token, scope) || this;
        _this.injectable = injectable;
        return _this;
    }
    FactoryInjector.prototype.result = function (target) {
        return this.injectFunction(this.injectable, target);
    };
    return FactoryInjector;
}(AbstractCachedInjector));
var ClassInjector = /** @class */ (function (_super) {
    tslib_1.__extends(ClassInjector, _super);
    function ClassInjector(parent, token, scope, injectable) {
        var _this = _super.call(this, parent, token, scope) || this;
        _this.injectable = injectable;
        return _this;
    }
    ClassInjector.prototype.result = function (_target) {
        return this.injectClass(this.injectable);
    };
    return ClassInjector;
}(AbstractCachedInjector));
exports.rootInjector = new RootInjector();
//# sourceMappingURL=InjectorImpl.js.map